@@ -32,7 +32,6 @@ type 'a alpha_term = {
   sz : int;
 }
 
-
 type 'a simple_term_aux = 
   | Cte of bool 
   | Other of 'a xterm
@@ -68,12 +67,12 @@ module Make(X : ALIEN) = struct
   let is_mine_a _ = false
 
   let is_mine_symb = function
-    | Sy.Bitv _ | Sy.Binop Sy.At -> true
-    | s -> Sy.equal s Sy.shat
+    | Sy.Bitv _ | Sy.Op (Sy.Concat | Sy.Extract)  -> true
+    | _ -> false
 
   let is_mine_type _ = assert false
 
-  let existential = function
+  let unsolvable = function
       [{bv =Other (Var _)}] -> true
     | _ -> false
 	
@@ -223,12 +222,11 @@ module Make(X : ALIEN) = struct
     let make t =
       let rec make_rec t' ctx = match T.view t' with
 	| {T.f = Sy.Bitv s } -> string_to_bitv s, ctx
-	| {T.f = Sy.Binop Sy.At ; xs = [t1;t2] ; ty = Ty.Tbitv n} ->
+	| {T.f = Sy.Op Sy.Concat ; xs = [t1;t2] ; ty = Ty.Tbitv n} ->
 	    let r1, ctx = make_rec t1 ctx in
 	    let r2, ctx = make_rec t2 ctx in
 	    { bv = I_Comp (r1, r2) ; sz = n }, ctx
-	| {T.f = s; xs = [t1;ti;tj] ; ty = Ty.Tbitv n} 
-	    when Sy.equal s Sy.shat ->
+	| {T.f = Sy.Op Sy.Extract; xs = [t1;ti;tj] ; ty = Ty.Tbitv n} ->
 	    begin 
 	      match T.view ti , T.view tj with
 		| { T.f = Sy.Int i } , { T.f = Sy.Int j } -> 
@@ -619,7 +617,23 @@ module Make(X : ALIEN) = struct
 	  let c = compare_simple_term st1 st2 in
 	  if c<>0 then c else comp l1 l2
     in comp b1 b2
-    
+
+  let hash_xterm = function
+    | Var {var = i; sorte = A} -> 11 * i
+    | Var {var = i; sorte = B} -> 17 * i
+    | Var {var = i; sorte = C} -> 19 * i
+    | Alien r -> 23 * X.hash r
+
+  let hash_simple_term_aux = function
+    | Cte b -> 11 * Hashtbl.hash b
+    | Other x -> 17 * hash_xterm x
+    | Ext (x, a, b, c) -> 
+	hash_xterm x + 19 * (a + b + c)
+      
+  let hash l = 
+    List.fold_left 
+      (fun acc {bv=r; sz=sz} -> acc + 19 * (sz + hash_simple_term_aux r) ) 19 l
+
   let leaves bitv = 
     List.fold_left 
       (fun acc x -> 
@@ -686,7 +700,7 @@ module Make(X : ALIEN) = struct
 
 
   (* ne resout pas quand c'est deja resolu *)
-  let solve repr u t = 
+  let solve u t = 
     if Options.debug_bitv then 
       eprintf "[Bitv] solve %a = %a@." X.print u X.print t;
     
@@ -745,70 +759,19 @@ module Make(X : ALIEN) = struct
       let compare = compare
     end)
 
+  let fully_interpreted sb = true
+
   module Rel = struct
 
     type r = X.r
-        (* ces X.t sont toujours des constantes *)
-        (* Cette map associe à chaque représentant r de type bitvector
-           une constante c tel que toutes les constantes plus petite
-           que c sont différentes de r.
-           Un représentant r est associé à None si c'est une constante*)
-    type elt = t
-    type t =  (elt option) M.t
+    type t =  unit
 
-    exception Inconsistent    
-    let empty () = M.empty
-
-    let is_bitv t = 
-      match (Term.view t).T.ty with 
-        | Ty.Tbitv _ -> true
-        | _ -> false
-
-    let assume env la = 
-      let leqs = 
-	List.fold_left 
-          (fun acc (a, root) -> 
-             match a with
-               | Literal.Neq(rx1,rx2) ->
-                   begin
-		     match X.extract rx1, X.extract rx2 with
-                       | Some [r1], Some [r2] when r1.sz = 1 -> 
-                           begin
-			     match r1.bv, r2.bv with
-                               | Cte b, Other o -> 
-				   let v = X.embed [r1] in
-				   (Literal.Eq(rx2,v),root)::acc  
-			       | Other o, Cte b -> 
-				   let v = X.embed [r2] in
-				   (Literal.Eq(rx1,v),root)::acc  
-                               | _ -> acc
-			   end
-                       | _ -> acc
-                   end
-               | _ -> acc) [] la
-      in 
-      env, leqs
- 
-    let cst0 n = 
-      let rec aux acc = function
-        | n when n <= 0 -> acc
-        | n -> aux ({bv=Cte false;sz=1}::acc) (n-1) in
-      aux [] n
-        
-    let add env r = 
-      match X.type_info r with
-        | Ty.Tbitv i when not (M.mem r env) ->     
-            let cst0 = Some (cst0 i) in
-            M.add r cst0 env
-        | _ -> env
-
-    let rec succ = function
-      | [] -> raise Exception.Inconsistent
-      | ({bv=false} as x)::l -> {x with bv = true }::l
-      | ({bv=true } as x)::l -> {x with bv = false}::(succ l)
-
-    let case_split env = env, []    
-    let query _ _ = false
+    let empty () = ()
+    let assume _ _ _ = (), []
+    let add _ _ = ()
+    let case_split _ = []    
+    let query _ _ _ = Sig.No
+    let instantiate _ _ _ _ _ = (), []
   end
 
 end
