@@ -15,4 +15,6 @@
 (*                                                                        *)
 (**************************************************************************)
 
-val make : Why_ptree.quant_form -> Why_ptree.tform
+open Why_ptree
+
+val make : int quant_form -> int tform
