@@ -17,16 +17,26 @@
 
 type t
 
-val everything : t
+(* val everything : t *)
 
 val empty : t
 
-val singleton : Formula.t -> t
+val mem_as_bj : Formula.t -> t -> bool
 
-val make : Formula.Set.t -> t
+val singleton : ?bj:bool -> Formula.t -> t
+
+val make_deps : Formula.Set.t -> t
 
 val union : t -> t -> t
 
+val merge : t -> t -> t
+
 val remove : Formula.t -> t -> t
 
-val compare : t -> t -> int
+val print : Format.formatter -> t -> unit
+
+val print_proof : Format.formatter -> t -> unit
+
+val ids_of : t -> int list option
+
+val formulas_of : t -> Formula.Set.t
