@@ -17,48 +17,51 @@
 
 type t
 
-type lemma = {
-  vars: Symbols.Set.t;  (* toplevel quantified variables *)
-  trs : Term.t list list; (* multi-triggers *)
-  f : t;  (* the main lemma's formula *)
-  name : string
-}
-
+type lemma =
+    { qvars: Symbols.Set.t;  (* toplevel quantified variables *)
+      triggers : Term.t list list; (* multi-triggers *)
+      main : t;  (* the main lemma's formula *)
+      name : string; }
+      
 and llet = {
-  lvar: Symbols.t;
-  lsubst : Term.t Subst.t;
-  lsubst_ty : Ty.subst;
-  lterm : Term.t;
-  lf : t;
+  let_var: Symbols.t;
+  let_subst : Term.subst;
+  let_term : Term.t;
+  let_f : t;
 }
 
-and ssko = {
-  ssubst : Term.t Subst.t;
-  ssubst_ty : Ty.subst;
-  sf : t;
+and skolem = {
+  sko_subst : Term.subst;
+  sko_f : t;
 }
 
 and view = 
-    U of t list   (* unit clauses *)
-  | C of t*t      (* a clause (t1 or t2) *)
-  | Lit of Literal.t   (* an atom *)
-  | Lem of lemma   (* a lemma *)
-  | Sko of ssko  (* lazy substitution *)
+    Unit of t*t  (* unit clauses *)
+  | Clause of t*t      (* a clause (t1 or t2) *)
+  | Literal of Literal.LT.t   (* an atom *)
+  | Lemma of lemma   (* a lemma *)
+  | Skolem of skolem  (* lazy substitution *)
   | Let of llet (* a binding of a term *)
 
 val mk_not : t -> t
-val mk_and : t -> t -> t
-val mk_or : t -> t -> t
-val mk_imp : t -> t -> t
-val mk_if : Term.t -> t -> t -> t
-val mk_iff : t -> t -> t
-val mk_lit : Literal.t -> t
-val mk_forall : Term.Set.t -> Term.Set.t -> Term.t list list -> t -> string -> t
-val mk_exists : Term.Set.t -> Term.Set.t -> Term.t list list -> t -> string -> t
-val mk_let : Term.Set.t -> Symbols.t -> Term.t -> t -> t
+val mk_and : t -> t -> int -> t
+val mk_or : t -> t -> int -> t
+val mk_imp : t -> t -> int -> t
+val mk_if : Term.t -> t -> t -> int -> t
+val mk_iff : t -> t -> int -> t
+val mk_lit : Literal.LT.t -> int -> t
+val mk_forall : Term.Set.t -> Term.Set.t -> Term.t list list -> t ->
+  string -> int -> t
+val mk_exists : Term.Set.t -> Term.Set.t -> Term.t list list -> t ->
+  string -> int -> t
+val mk_let : Term.Set.t -> Symbols.t -> Term.t -> t -> int -> t
+
+val add_label : Hstring.t -> t -> unit
+val label : t -> Hstring.t
 
 val view : t -> view
 val size : t -> int
+val id : t -> int
 
 val print : Format.formatter -> t -> unit
 
