@@ -15,40 +15,59 @@
 (*                                                                        *)
 (**************************************************************************)
 
+
+module type OrderedType = sig
+  type t
+  val compare : t -> t -> int
+  val hash :  t -> int
+  val print : Format.formatter -> t -> unit
+end
+
 type 'a view = 
-    Eq of 'a * 'a 
-  | Neq of 'a * 'a 
+  | Eq of 'a * 'a 
+  | Distinct of bool * 'a list
   | Builtin of bool * Hstring.t * 'a list
 
-type t
+module type S = sig
+  type elt
+  type t
 
-val make : Term.t view -> t
-val mk_pred : Term.t -> t
+  val make : elt view -> t
+  val view : t -> elt view
 
-val vrai : t
-val faux : t 
+  val neg : t -> t
 
-val neg_view : 'a view -> 'a view
+  val add_label : Hstring.t -> t -> unit
+  val label : t -> Hstring.t
 
-val view : t -> Term.t view
-val compare : t -> t -> int
-val equal : t -> t -> bool
-val hash : t -> int
+  val print : Format.formatter -> t -> unit
 
-val neg : t -> t
-val apply_subst : Term.subst -> t -> t
+  val compare : t -> t -> int
+  val equal : t -> t -> bool
+  val hash : t -> int
 
-val terms_of : t -> Term.Set.t
-val vars_of : t -> Symbols.Set.t
+  module Map : Map.S with type key = t
+  module Set : Set.S with type elt = t
+
+end
+
+module Make ( X : OrderedType ) : S with type elt = X.t
+
+module type S_Term = sig
+
+  include S with type elt = Term.t
+
+  val mk_pred : Term.t -> t
+  val vrai : t
+  val faux : t
+
+  val apply_subst : Term.subst -> t -> t
+
+  val terms_of : t -> Term.Set.t
+  val vars_of : t -> Symbols.Set.t
+
+end
+
+module LT : S_Term
 
 
-val print_view : 
-  (Format.formatter -> 'a -> unit) -> Format.formatter -> 'a view -> unit
-
-val print : Format.formatter -> t -> unit
-val print_list : Format.formatter -> t list-> unit
-
-module SetEq : Set.S with type elt = t * Term.t * Term.t
-module Map : Map.S with type key = t
-module Set : Set.S with type elt = t
-
