@@ -46,7 +46,7 @@ module type X = sig
   type t
 
   val class_of : t -> Term.t -> Term.t list
-  val query : Literal.t -> t -> bool
+  val query : Literal.LT.t -> t -> Sig.answer
 
 end
 
@@ -148,23 +148,37 @@ module Make (X : X) = struct
   let add_msymb uf f t ({sbt=(s_t,s_ty)} as sg)= 
     try 
       let t' = SubstT.find f s_t in
-      if X.query (Literal.make(Literal.Eq(t,t'))) uf then sg 
+      let a = Literal.LT.make (Literal.Eq (t, t')) in
+      if X.query a uf (*Explanation.singleton (Formula.mk_lit a)*) <> Sig.No
+      then sg 
       else raise Echec
     with Not_found ->  {sg with sbt=(SubstT.add f t s_t,s_ty) }
 
-  let rec iter_exception f m = function
-      [] -> raise Echec
-    | x::l -> try f m x with Echec -> iter_exception f m l
+(* ancien iter_exception: pas complet
+
+  let rec iter_exception f gsb l = match l with
+      []    -> raise Echec
+    | xs::l -> try (f gsb xs) with Echec -> iter_exception f gsb l
+*)	
+
+  let rec iter_exception f gsb l =
+    let l = 
+      List.fold_left
+        (fun acc xs -> try (f gsb xs) @ acc with Echec -> acc) [] l in
+    match l with [] -> raise Echec | l  -> l
 	
   let rec matchterm env uf ( {sbt=(s_t,s_ty);gen=g;goal=b} as sg) pat t =
     let {T.f=f_pat;xs=pats;ty=ty_pat} =  T.view pat in
     match f_pat with
 	Symbols.Var _ -> 
-	  (try
-	     let s_ty = Ty.matching s_ty ty_pat (T.view t).T.ty in
-	     let g',b' = infos max (||) t g b env in
-	     add_msymb uf f_pat t {sbt=(s_t,s_ty);gen=g';goal=b'}
-	   with Ty.TypeClash _ -> raise Echec)
+	  let sb =
+            (try
+	       let s_ty = Ty.matching s_ty ty_pat (T.view t).T.ty in
+	       let g',b' = infos max (||) t g b env in
+	       add_msymb uf f_pat t {sbt=(s_t,s_ty);gen=g';goal=b'}
+	     with Ty.TypeClash _ -> raise Echec)
+          in 
+          [sb]
       | _ -> 
 	  let l = List.map T.view (X.class_of uf t) in
 	  let s_ty , l = 
@@ -182,8 +196,13 @@ module Make (X : X) = struct
 	    (fun m {T.xs=xs} -> matchterms env uf m pats xs) 
 	    { sg with sbt = (s_t,s_ty)} l
 	  
+
   and matchterms env uf sg pats xs = 
-    try List.fold_left2 (matchterm env uf) sg pats xs 
+    try List.fold_left2 
+          (fun sb_l pt arg -> 
+            let sb_ll = List.map (fun sg -> matchterm env uf sg pt arg) sb_l in
+            List.flatten sb_ll
+          )[sg] pats xs 
     with Invalid_argument _ -> raise Echec
 
   let matchpat env uf pat_info lsubst ({gen=g; goal=b} as sg,pat) = 
@@ -197,7 +216,7 @@ module Make (X : X) = struct
 		 try 
 		   let gen , but = infos max (||) t g b env in
 		   (matchterms env uf
-			{sg with gen=gen ; goal=but } pats xs)::l 
+			{sg with gen=gen ; goal=but } pats xs) @ l 
 		 with Echec -> l)
 	      (SubstT.find f env.fils) lsubst
 	  with Not_found -> lsubst
