@@ -1,17 +1,23 @@
 (**************************************************************************)
 (*                                                                        *)
 (*     The Alt-ergo theorem prover                                        *)
-(*     Copyright (C) 2006-2008                                            *)
+(*     Copyright (C) 2006-2010                                            *)
 (*                                                                        *)
 (*     Sylvain Conchon                                                    *)
 (*     Evelyne Contejean                                                  *)
-(*     CNRS-LRI-Universite Paris Sud                                      *)
+(*     Stephane Lescuyer                                                  *)
+(*     Mohamed Iguernelala                                                *)
+(*     Alain Mebsout                                                      *)
+(*                                                                        *)
+(*     CNRS - INRIA - Universite Paris Sud                                *)
 (*                                                                        *)
 (*   This file is distributed under the terms of the CeCILL-C licence     *)
 (*                                                                        *)
 (**************************************************************************)
 
-val print_term : Format.formatter -> Why_ptree.tterm -> unit
-val print_term_list : Format.formatter -> Why_ptree.tterm list -> unit
+open Why_ptree
 
-val print_form : Format.formatter -> Why_ptree.tform -> unit
+val print_term : Format.formatter -> ('a tterm, 'a) annoted -> unit
+val print_term_list : Format.formatter -> ('a tterm, 'a) annoted list -> unit
+
+val print_form : Format.formatter -> ('a tform, 'a) annoted -> unit
