@@ -201,6 +201,6 @@ let rec print_list sep print_ele fmt = function
   |[] -> ()
   |[a] -> print_ele fmt a
   |a::l -> 
-     Format.fprintf fmt "%a@ %s %a" print_ele a sep (print_list sep print_ele) l
+     Format.fprintf fmt "%a %s %a" print_ele a sep (print_list sep print_ele) l
        
 (* Pour l'extérieur *)
