@@ -15,5 +15,8 @@
 (*                                                                        *)
 (**************************************************************************)
 
+open Why_ptree
+
 val split_and_prune :
-  int -> Why_ptree.typed_decl list -> (Why_ptree.typed_decl * bool) list list
+  int -> (int tdecl, int) annoted list -> 
+  ((int tdecl,int) annoted * bool) list list
