@@ -17,6 +17,7 @@
 
 open Options
 open Format
+open Sig
 
 module A = Literal
 module CcX = Cc.Make(Combine.CX)
@@ -26,6 +27,8 @@ module SF = F.Set
 module MF = F.Map
 module Ex = Explanation
 
+let steps = ref 0L
+
 type gformula = { 
   f: F.t; 
   age: int; 
@@ -43,8 +46,8 @@ type t = {
     matching : M.t
 }
       
-exception Sat
-exception Unsat
+exception Sat of t
+exception Unsat of Ex.t
 exception I_dont_know
 exception IUnsat of Ex.t
 
@@ -52,38 +55,41 @@ let max_max_size = 96
 
 module Print = struct
 
-  let assume {f=f;age=age;name=lem;mf=mf} = 
+  let assume {f=f;age=age;name=lem;mf=mf} dep = 
     if debug_sat then
-      match F.view f with
-	  F.U _ -> ()
-	    
-	| F.C _ -> 
-	    printf "@[@{<C.Bold>[sat]@}";
-	    printf "@{<C.G_Cyan>I assume a clause@} @[%a@]@]@." F.print f
+      begin
+	(match F.view f with
+	    F.Unit _ -> ()
 	      
-	| F.Lem _ ->
-	    printf "@[@{<C.Bold>[sat]@}";
-	    printf "@{<C.G_Yellow>I assume a [%d-atom] lemma@}" (F.size f);
-	    printf "@[%a@]@]@." F.print f
-	      
-	| F.Lit a -> 
-	    let s = 
-	      match lem with 
-		  None -> "" 
-		| Some ff -> 
-		    (match F.view ff with F.Lem xx -> xx.F.name | _ -> "") 
-	    in
-	    printf "\n@[@{<C.Bold>[sat]@}";
-	    printf "@{<C.G_Blue_B>I assume a literal@}";
-	    printf "(%s) %a@]@." s Literal.print a;
-	    printf "================================================@.@."
-	      
-	| F.Sko{F.ssubst=s;ssubst_ty=s_ty;sf=f} ->
-	    printf "@[@{<C.Bold>[sat]@} I assume a skolem %a @]@." F.print f 
-	      
-	| F.Let {F.lvar=lvar;lterm=lterm;lsubst=lsubst;lf=lf} ->
-	    printf "@[@{<C.Bold>[sat]@} I assume a let %a =@ %a in@ %a@ @]@." 
-              Symbols.print lvar Term.print lterm F.print lf
+	  | F.Clause _ -> 
+	      printf "@[@{<C.Bold>[sat]@}";
+	      printf "@{<C.G_Cyan>I assume a clause@} @[%a@]@]@." F.print f
+		
+	  | F.Lemma _ ->
+	      printf "@[@{<C.Bold>[sat]@}";
+	      printf "@{<C.G_Yellow>I assume a [%d-atom] lemma: @}" (F.size f);
+	      printf "@[%a@]@]@." F.print f
+		
+	  | F.Literal a -> 
+	      let s = 
+		match lem with 
+		    None -> "" 
+		  | Some ff -> 
+		      (match F.view ff with F.Lemma xx -> xx.F.name | _ -> "") 
+	      in
+	      printf "\n@[@{<C.Bold>[sat]@}";
+	      printf "@{<C.G_Blue_B>I assume a literal@}";
+	      printf "(%s) %a@]@." s Literal.LT.print a;
+	      printf "================================================@.@."
+		
+	  | F.Skolem{F.sko_subst=(s,s_ty); sko_f=f} ->
+	      printf "@[@{<C.Bold>[sat]@} I assume a skolem %a @]@." F.print f 
+		
+	  | F.Let {F.let_var=lvar; let_term=lterm; let_f=lf} ->
+	      printf "@[@{<C.Bold>[sat]@} I assume a let %a =@ %a in@ %a@ @]@." 
+		Symbols.print lvar Term.print lterm F.print lf);
+	printf " with explanations : %a@." Explanation.print dep
+      end
 
   let unsat () = 
     if debug_sat then printf "@[@{<C.Bold>[sat]@} @{<C.G_Red_B>unsat@}@]@."
@@ -103,13 +109,31 @@ module Print = struct
 
   let backjumping f = 
     if debug_sat then 
-      (printf "@[@{<C.Bold>[sat]@} @{<C.G_Green>I dont' consider the case@}";
+      (printf "@[@{<C.Bold>[sat]@} @{<C.G_Green>I don't consider the case @}";
        printf "%a@]@." F.print f)
        
   let elim _ _ = if debug_sat && verbose then printf "@[@{<C.Bold>[elim]@}@."
 
   let red _ _ = if debug_sat && verbose then printf "@[@{<C.Bold>[red]@}@."
 
+  let delta d = 
+    if debug_sat && verbose then begin
+      printf "@[@{<C.Bold>[sat]@} - Delta ---------------------]@.";
+      List.iter (fun (f1, f2, ex) ->
+	printf "(%a or %a), %a@." F.print f1.f F.print f2.f Ex.print ex) d;
+      printf "@[@{<C.Bold>[sat]@} --------------------- Delta -]@."
+    end
+    
+  let gamma g =
+    if debug_sat then begin
+      printf "@[@{<C.Bold>[sat]@} - GAMMA ---------------------]@.";
+      MF.iter (fun f ex ->
+	printf "%a \t->\t%a@." F.print f Ex.print ex) g;
+      printf "@[@{<C.Bold>[sat]@} --------------------- GAMMA -]@."
+      
+    end
+      
+
 end
 
 (* matching part of the solver *)
@@ -133,43 +157,6 @@ let rec double_until min s =
   let s2 = s + b_max_size in 
     if s2 >= min then s2 else double_until min s2
 
-(*
-let mtriggers mode env lem (age,dep) (max_size,acc) =
-  let size = F.size lem in
-  match F.view lem with
-      F.Lem {F.trs=tgs;F.f=f;name=name} ->
-	let next_max_size =
-	  if size > max_size then
-	    match acc with
-	      | [] -> double_until size max_size
-	      | _ -> raise (EnoughLemmasAlready (max_size,acc))
-	  else max_size
-	in
-	List.fold_left
-	  (fun (sz,acc) pat ->
-	     let ll = M.matching lem pat env.matching env.tbox in
-	     let (sz1, ll1) =
-	       List.fold_left
-		 (fun (sz,acc) {Matching.sbt=s;gen=g;goal=b} ->
-		    if mode && not b then (sz,acc)
-		    else
-		      let nf = F.apply_subst s f in
-		      if MF.mem nf env.gamma then (sz,acc) else
-			(if debug_sat_simple then
-			   Format.printf "we keep %s [%d-atom]@." name size;
-			 let p =
-			   {f=nf;age=1+(max g age);name=Some lem;mf=true;gf=b}
-			 in
-			 (sz,(p,dep)::acc))
-		 )
-		 (sz,acc) ll
-	     in
-	     sz1 , ll1
-	  )
-	  (next_max_size, acc) tgs
-    | _ -> assert false
-*)
-
 let mtriggers env formulas max_size = 
   let stop = ref false in
   try
@@ -186,7 +173,7 @@ let mtriggers env formulas max_size =
 	 in
 	 let env = 
 	   match F.view lem with
-	       F.Lem {F.trs=tgs; f=f} -> 
+	       F.Lemma {F.triggers = tgs; main = f} -> 
 		 List.fold_left 
 		   (fun env tg ->
 		      let info = 
@@ -225,61 +212,96 @@ let new_facts mode env =
     )
     [] (M.query env.matching env.tbox)
 
-let mround predicate mode env max_size = 
-  Print.mround max_size;
-  let axioms = if predicate then env.definitions else env.lemmas in
-  let env, max_size = mtriggers env axioms max_size in
-  let rec bouclage n (env, lf) = 
-    if n <=0 then (env, lf)
-    else
-      let env = 
-	List.fold_left 
-	(fun env (f,_) -> add_terms env (F.terms f.f) mode f.age None)
-	env lf
-      in
-      bouclage (n-1) (env, (new_facts mode env))
-  in
-  let _, lf = bouclage 1 (env, []) in
-  max_size, lf
-    
-(*
-let mround mode env max_size = 
-  Print.mround max_size;
-  try MF.fold (mtriggers mode env) env.lemmas (max_size,[])
-  with EnoughLemmasAlready (max,l) -> (max,l)
 
-let mround_predicate mode env max_size = 
-  SF.fold 
-    (fun f -> mtriggers mode env f (0,Ex.empty)) env.definitions (max_size,[])
-*)
+let mround predicate mode env max_size =
+  let round mode =
+    Print.mround max_size;
+    let axioms = if predicate then env.definitions else env.lemmas in
+    let env, max_size = mtriggers env axioms max_size in
+    let rec bouclage n (env, lf) = 
+      if n <=0 then (env, lf)
+      else 
+        let env = 
+	  List.fold_left 
+	    (fun env (f,_) -> add_terms env (F.terms f.f) mode f.age None)
+	    env lf
+        in
+        bouclage (n-1) (env, (new_facts mode env))
+    in
+    let _, lf = bouclage Options.bouclage (env, []) in
+    max_size, lf 
+  in
+  let max_size, lf = round (mode || Options.goal_directed) in 
+  if Options.goal_directed && lf = [] then round false 
+  else max_size, lf
   
+
+let extract_model t = 
+  let s = ref SF.empty in
+  MF.iter 
+    (fun f _ -> 
+       let lbl = F.label f in
+       if not (Hstring.equal Hstring.empty lbl) then
+	 s := SF.add f !s
+    ) 
+    t.gamma;
+  !s
+
+let print_model fmt s = 
+  SF.iter (fprintf fmt "%a\n" F.print) s
+
 (* sat-solver *)
 
 let elim {f=f} env = 
   MF.mem f env.gamma ||
     match F.view f with 
-	F.Lit a -> CcX.query a (CcX.add a env.tbox)  
+      | F.Literal a -> CcX.query a env.tbox <> No
       | _ -> false
 
-let size_formula = 1000
+let size_formula = 1_000_000
 
 let red {f=f} env = 
-  let f = F.mk_not f in
+  let nf = F.mk_not f in
   try 
-    Some(MF.find f env.gamma)
+    Yes (MF.find nf env.gamma)
   with Not_found -> 
-    match F.view f with
-	F.Lit a -> 
-	  let tbox = CcX.add a env.tbox in
-	  if CcX.query a tbox then Some (CcX.explain a tbox) else None
-      | _ -> None
+    match F.view nf with
+      |	F.Literal a -> CcX.query a env.tbox
+      | _ -> No
 
 let pred_def env f = 
+  let ff = {f=f;age=0;name=None;mf=false;gf=false} in
+  Print.assume ff Explanation.empty;
   { env with definitions = MF.add f (0,Ex.empty) env.definitions }
 
+
+
+let add_dep f dep =
+  match F.view f with 
+    | F.Literal _ when proof -> 
+      if not (Ex.mem_as_bj f dep) then
+	Ex.union (Ex.singleton ~bj:false f) dep
+      else dep
+    | _ -> dep
+  
+
+let rec add_dep_of_formula f dep =
+  let dep = add_dep f dep in
+  match F.view f with 
+    | F.Unit (f1, f2) when proof ->
+      add_dep_of_formula f2 (add_dep_of_formula f1 dep)
+    | F.Clause _ when proof -> 
+	Ex.union (Ex.singleton ~bj:false f) dep
+    | _ -> dep
+
+
 let rec assume env ({f=f;age=age;name=lem;mf=mf;gf=gf} as ff ,dep) =
   try
-    (try raise (IUnsat (Ex.union dep (MF.find (F.mk_not f) env.gamma)))
+    let dep = add_dep f dep in
+    let dep_gamma = add_dep_of_formula f dep in
+    (try (* Print.gamma env.gamma; *)
+	 (* fprintf fmt "ass:%a %a @." F.print (F.mk_not f) Ex.print dep_gamma; *)
+       raise (IUnsat (Ex.union dep_gamma (MF.find (F.mk_not f) env.gamma)))
      with Not_found -> ());
     if MF.mem f env.gamma then env
     else 
@@ -290,20 +312,22 @@ let rec assume env ({f=f;age=age;name=lem;mf=mf;gf=gf} as ff ,dep) =
 	  let env =
 	    if mf && glouton  && size < size_formula then 
 	      add_terms env (F.terms f) gf age lem else env in
-	  let env = { env with gamma = MF.add f dep env.gamma } in
-	  Print.assume ff;
+	  let env = { env with gamma = MF.add f dep_gamma env.gamma } in
+	  Print.assume ff dep;
 	  match F.view f with
-	      F.U l -> 
-		List.fold_left assume env 
-		  (List.map (fun x->{f=x;age=age;
-				     name=lem;mf=mf;gf=gf},dep) l)
-
-	    | F.C(f1,f2) -> 
+	    | F.Unit (f1, f2) ->
+	      let env = assume env 
+		({ f = f1; age = age; name = lem; mf = mf; gf = gf }, dep) in
+	      assume env 
+		({ f = f2; age = age; name = lem; mf = mf; gf = gf }, dep) 
+	    | F.Clause(f1,f2) -> 
+	        (* let dep = Ex.union (Ex.singleton ~bj:false f) dep in *)
 		let p1 = {f=f1;age=age;name=lem;mf=mf;gf=gf} in
 		let p2 = {f=f2;age=age;name=lem;mf=mf;gf=gf} in
 		bcp { env with delta = (p1,p2,dep)::env.delta }
 
-	    | F.Lem _ ->
+	    | F.Lemma _ ->
+	        (* let dep = Ex.union (Ex.singleton ~bj:false f) dep in *)
 		let age , dep = 
 		  try 
 		    let age' , dep' = MF.find f env.lemmas in
@@ -312,30 +336,43 @@ let rec assume env ({f=f;age=age;name=lem;mf=mf;gf=gf} as ff ,dep) =
 		in
 		bcp { env with lemmas=MF.add f (age,dep) env.lemmas }
 
-	    | F.Lit a -> 
+	    | F.Literal a ->
+	        (* let dep = Ex.union (Ex.singleton ~bj:false f) dep in *)
 		let env = 
 		  if mf && size < size_formula then 
-		    add_terms env (A.terms_of a) gf age lem
+		    add_terms env (A.LT.terms_of a) gf age lem
 		  else env 
 		in
-		let env = { env with tbox = CcX.assume a dep env.tbox } in
+		let tbox, cpt = CcX.assume a dep env.tbox in
+		steps := Int64.add (Int64.of_int cpt) !steps;
+		if stepsb <> -1 
+		  && Int64.compare !steps (Int64.of_int stepsb) > 0 then 
+		  begin 
+		    printf "Steps limit reached: %Ld@." !steps;
+		    exit 1
+		  end;
+		let env = { env with tbox = tbox } in
 		bcp env
 
-	    | F.Sko{F.ssubst=s;ssubst_ty=s_ty;sf=f} -> 
-		let f' = F.apply_subst (s,s_ty) f in
+	    | F.Skolem{F.sko_subst=sigma; sko_f=f} -> 
+		let f' = F.apply_subst sigma f in
 		assume env ({f=f';age=age;name=lem;mf=mf;gf=gf},dep)
 
-            | F.Let {F.lvar=lvar;lterm=lterm;lsubst=lsubst;lf=lf} ->
-                let f' = F.apply_subst (lsubst,Ty.esubst) lf in
-                let v = Symbols.Map.find lvar lsubst in
-                let env = assume env ({f=F.mk_lit (A.make (A.Eq(v,lterm)));
-				       age=age;name=lem;mf=mf;gf=gf},dep) 
+            | F.Let {F.let_var=lvar; let_term=lterm; let_subst=s; let_f=lf} ->
+                let f' = F.apply_subst s lf in
+		let id = F.id f' in
+                let v = Symbols.Map.find lvar (fst s) in
+                let env = assume env 
+		  ({f=F.mk_lit (A.LT.make (A.Eq(v,lterm))) id;
+		    age=age;name=lem;mf=mf;gf=gf},dep) 
 		in
                 assume env ({f=f';age=age;name=lem;mf=mf;gf=gf},dep)
       end
-  with Exception.Inconsistent -> raise (IUnsat Ex.everything)
+  with Exception.Inconsistent expl -> 
+    if debug_sat then fprintf fmt "inconsistent %a@." Ex.print expl; 
+    raise (IUnsat expl)
     
-and bcp env = 
+and bcp env =
   let cl , u = 
     List.fold_left 
       (fun (cl,u) ((f1,f2,d) as fd) -> 
@@ -344,11 +381,11 @@ and bcp env =
 	 else 
            (Print.red f1 f2;
 	   match red f1 env with
-	       Some d1 -> (cl,(f2,Ex.union d d1)::u)
-	     | None -> 
+	     | Yes d1 -> (cl,(f2,Ex.union d d1)::u)
+	     | No -> 
 		 match red f2 env with
-		     Some d2 -> (cl,(f1,Ex.union d d2)::u)
-		   | None -> fd::cl , u)
+		     Yes d2 -> (cl,(f1,Ex.union d d2)::u)
+		   | No -> fd::cl , u)
       ) ([],[]) env.delta
   in
   List.fold_left assume {env with delta=cl} u
@@ -359,27 +396,30 @@ let rec unsat_rec env fg stop max_size =
     back_tracking (assume env fg) stop max_size
   with IUnsat d-> Print.unsat (); d
 
-and back_tracking env stop max_size = match env.delta with
+and back_tracking env stop max_size =  match env.delta with
     []  when stop >= 0  -> 
       let _ , l2 = mround true false env max_max_size in 
       let env = List.fold_left assume env l2 in
+
       let max_size , l1 = mround false false env max_size in 
       let env = List.fold_left assume env l1 in
 
-(*      let _ , l2' = mround_predicate true false env max_max_size in 
-      let env = List.fold_left assume env l2' in
-      let max_size , l1' = mround false env max_size in 
-      let env = List.fold_left assume env l1' in*)
-
       let env = 
 	List.fold_left 
 	  (fun env ({f=f; age=g; name=lem; gf=gf},_) -> 
 	     add_terms env (F.terms f) gf g lem) env l1 
       in
-      (match l1,l2 with
-	   [] , [] -> 
-	     raise Sat
-	 | l1,l2 -> 
+      (match l1, l2 with
+	 | [], [] -> 
+	     let m = extract_model env in
+	     if all_models then 
+	       begin
+		 Format.printf "--- SAT ---\n";
+		 Format.printf "%a@." print_model m;
+		 raise (IUnsat (Ex.make_deps m))
+	       end;
+	     raise (Sat env)
+	 | l1, l2 -> 
 	     back_tracking 
 	       (List.fold_left assume  (List.fold_left assume env l2) l1) 
 	       (stop-1) (max_size + b_max_size))
@@ -387,13 +427,13 @@ and back_tracking env stop max_size = match env.delta with
       raise I_dont_know
   | ({f=f;age=g;name=lem;mf=mf} as a,b,d)::l -> 
       Print.decide f;
-      let dep = unsat_rec {env with delta=l} (a,Ex.singleton f) 
-	stop max_size in
+      let dep = unsat_rec {env with delta=l} (a,Ex.singleton f) stop max_size in
+      if debug_sat then fprintf fmt "unsat_rec : %a@." Ex.print dep;
       try
 	let dep' = Ex.remove f dep in
 	Print.backtracking (F.mk_not f);
 	unsat_rec
-	  (assume {env with delta=l} (b,Ex.union d dep'))
+	  (assume {env with delta=l} (b, Ex.union d dep'))
 	  ({a with f=F.mk_not f},dep') stop max_size
       with Not_found -> Print.backjumping (F.mk_not f); dep 
 	
@@ -402,20 +442,17 @@ let unsat env fg stop =
     let env = assume env (fg,Ex.empty) in
     let env = add_terms env (F.terms fg.f) fg.gf fg.age fg.name in
 
-(*    let max_size , l = mround false env 1 in
-    let env = List.fold_left assume env l in
-*)
-    let _, l = mround true false env max_max_size in
+    let _ , l = mround true false env max_max_size in
     let env = List.fold_left assume env l in
 
     let _ , l = mround false true env max_max_size in
     let env = List.fold_left assume env l in
 
-    ignore(back_tracking env stop 100)
-  with IUnsat _ -> Print.unsat ()
+    back_tracking env stop 100
+  with IUnsat dep -> Print.unsat ();dep
 
 let assume env fg = 
-  try assume env (fg,Ex.empty) with IUnsat _ -> raise Unsat
+  try assume env (fg,Ex.empty) with IUnsat d -> raise (Unsat d)
 
 let empty = { 
   gamma = MF.empty;
@@ -426,6 +463,5 @@ let empty = {
   definitions = MF.empty
 } 
 
-
-
-
+let start () = steps := 0L
+let stop () = !steps
