@@ -25,11 +25,10 @@ type gformula = {
   gf: bool;
 }
 
-exception Sat
-exception Unsat
+exception Sat of t
+exception Unsat of Explanation.t
 exception I_dont_know
 
-
 (* the empty sat-solver context *)
 val empty : t
 
@@ -43,4 +42,7 @@ val pred_def : t -> Formula.t -> t
 (* [unsat env f size] checks the unsatisfiability of [f] in
    [env]. Raises I_dont_know when the proof tree's height reaches
    [size]. Raises Sat if [f] is satisfiable in [env] *)
-val unsat : t -> gformula -> int -> unit
+val unsat : t -> gformula -> int -> Explanation.t
+
+val start : unit -> unit
+val stop : unit -> int64
