@@ -16,7 +16,7 @@
 (**************************************************************************)
 
 type t
-type view = {f: Symbols.t ; xs: t list; ty: Ty.t}
+type view = private {f: Symbols.t ; xs: t list; ty: Ty.t}
 
 type subst = t Subst.t * Ty.subst
 
@@ -27,23 +27,32 @@ val shorten : t -> t
 
 val vrai : t
 val faux : t
-val unit : t
+val void : t
 
 val int : string -> t
-val rat : string -> t
+val real : string -> t
 val bitv : string -> Ty.t -> t
 
+val fresh_name : Ty.t -> t
+
 val is_int : t -> bool
-val is_rat : t -> bool
+val is_real : t -> bool
 
 val compare : t -> t -> int
 val equal : t -> t -> bool
 val hash : t -> int
 
 val vars_of : t -> Symbols.Set.t
+val vty_of : t -> Ty.Svty.t
+
 val pred : t -> t
 
 val apply_subst : subst -> t -> t
+val compare_subst : subst -> subst -> int
+val fold_subst_term : (Symbols.t -> t -> 'b -> 'b) -> subst -> 'b -> 'b
+
+val union_subst : subst -> subst -> subst
+
 val print : Format.formatter -> t -> unit
 val print_list : Format.formatter -> t list -> unit
 
