@@ -17,13 +17,15 @@
 
 type t = 
     | Tint
-    | Trat
+    | Treal
     | Tbool
+    | Tunit
     | Tvar of tvar
     | Tbitv of int
     | Text of t list * Hstring.t
-    | Tarray
-        
+    | Tfarray of t * t
+    | Tsum of Hstring.t * Hstring.t list
+
 and tvar = { v : int ; mutable value : t option }
 
 type subst
@@ -35,6 +37,7 @@ exception TypeClash of t*t
 val tunit : t
 
 val text : t list -> string -> t
+val tsum : string -> string list -> t
 
 val shorten : t -> t
 
@@ -58,3 +61,11 @@ val compare_subst : subst -> subst -> int
 
 val print : Format.formatter -> t -> unit
 val printl : Format.formatter -> t list -> unit
+
+module Svty : Set.S
+
+val vty_of : t -> Svty.t
+
+val monomorphize: t -> t
+
+val print_subst: Format.formatter -> subst -> unit
