@@ -21,7 +21,13 @@ module L = List
 module T = Term
 module S = Symbols
 module ST = T.Set
-module SA = Literal.Set
+(* module SA = Literal.LT.Set *)
+
+module SA = Set.Make(struct
+  type t = Literal.LT.t * Explanation.t
+  let compare (s1,_) (s2,_) = Literal.LT.compare s1 s2
+end)
+
 type elt = ST.t * SA.t
   
 module Make (X : Sig.X) = struct
@@ -52,7 +58,8 @@ module Make (X : Sig.X) = struct
     match lvs with
 	[]    -> ST.empty
       | x::ls -> 
-	  L.fold_left (fun acc y -> ST.inter (fst(find y g)) acc)
+	  L.fold_left 
+	    (fun acc y -> ST.inter (fst(find y g)) acc)
 	    (fst(find x g)) ls
 	    
   let up_close_up g p v = 
@@ -72,7 +79,8 @@ module Make (X : Sig.X) = struct
       
   let print g = 
     let sterms fmt = ST.iter (F.fprintf fmt "%a " T.print) in
-    let satoms fmt = SA.iter (F.fprintf fmt "%a " Literal.print ) in
+    let satoms fmt = SA.iter (fun (a,e) -> 
+      F.fprintf fmt "%a %a" Literal.LT.print a Explanation.print e) in
     F.fprintf fmt "@{<C.Bold>[use]@} gamma :\n";
     iter 
       (fun t (st,sa) -> 
